﻿using mailinator_csharp_client;
using mailinator_csharp_client.Models.Messages.Entities;
using mailinator_csharp_client.Models.Messages.Requests;
using mailinator_csharp_client.Models.Responses;

namespace TestProject1
{
    public class MailinatorProxy
    {
        private readonly MailinatorClient MailinatorClient;

        public MailinatorProxy(MailinatorConfig creds)
        {
            MailinatorClient = new(creds.MailinatorId);
        }

        public async Task<FetchInboxResponse> GetInbox(string emailId, string domain)
        {
            // Now get the "activiation email"
            FetchInboxResponse? fetchInboxResponse = null;

            // Wait for ~10 seconds for the "activation" email to arrive.
            for (int i = 0; i < 10; i++)
            {
                //Fetch Inbox and initial message info
                FetchInboxRequest fetchInboxRequest = new() { Domain = domain, Inbox = emailId, Skip = 0, Limit = 20, Sort = Sort.asc };
                fetchInboxResponse = await this.MailinatorClient.MessagesClient.FetchInboxAsync(fetchInboxRequest);

                if (fetchInboxResponse.Messages.Count > 0)
                {
                    // If we found a message, there's no need to continue looping.
                    break;
                }
                else
                {
                    // If we didn't find a message, let's briefly pause before trying again. 
                    Thread.Sleep(1000);
                }
            }

            // If we couldn't get the inbox or no messages arrived during the polling wait, we need to fail.
            Assert.IsNotNull(fetchInboxResponse, $"Could not fetch the inbox for {emailId} at domain {domain}");
            Assert.AreEqual(1, fetchInboxResponse.Messages.Count, $"Could not find any messages in the mailbox {emailId}");

            return fetchInboxResponse;
        }

        public async Task<FetchMessageResponse> GetMessage(FetchInboxResponse fetchInboxResponse, string emailId, string domain)
        {
            FetchMessageRequest fetchMessageRequest = new() { Domain = domain, Inbox = emailId, MessageId = fetchInboxResponse.Messages[0].Id };
            FetchMessageResponse fetchMessageResponse = await this.MailinatorClient.MessagesClient.FetchMessageAsync(fetchMessageRequest);

            return fetchMessageResponse;
        }
    }
}
